@@ -28,94 +28,55 @@
 #' In both cases, Bias can assume values between
 #' -1 and 1 and is 0 ideally.
 #'
-#' @param true_values A vector with the true observed values of size n
-#' @param predictions nxN matrix of predictive samples, n (number of rows) being
-#' the number of data points and N (number of columns) the
-#' number of Monte Carlo samples
 #' @return vector of length n with the biases of the predictive samples with
 #' respect to the true values.
-#' @author Nikos Bosse \email{nikosbosse@gmail.com}
+#' @inheritParams ae_median_sample
+#' @author Nikos Bosse \email{nikosbosse@@gmail.com}
 #' @examples
 #'
 #' ## integer valued forecasts
 #' true_values <- rpois(30, lambda = 1:30)
 #' predictions <- replicate(200, rpois(n = 30, lambda = 1:30))
-#' bias(true_values, predictions)
+#' bias_sample(true_values, predictions)
 #'
 #' ## continuous forecasts
 #' true_values <- rnorm(30, mean = 1:30)
 #' predictions <- replicate(200, rnorm(30, mean = 1:30))
-#' bias(true_values, predictions)
-#'
-#'
+#' bias_sample(true_values, predictions)
 #' @export
 #' @references
 #' The integer valued Bias function is discussed in
-#' Assessing the performance of real-time epidemic forecasts: A case study of Ebola in the Western Area region of Sierra Leone, 2014-15
-#' Funk S, Camacho A, Kucharski AJ, Lowe R, Eggo RM, et al. (2019) Assessing the performance of real-time epidemic forecasts: A case study of Ebola in the Western Area region of Sierra Leone, 2014-15. PLOS Computational Biology 15(2): e1006785. https://doi.org/10.1371/journal.pcbi.1006785
+#' Assessing the performance of real-time epidemic forecasts: A case study of
+#' Ebola in the Western Area region of Sierra Leone, 2014-15 Funk S, Camacho A,
+#' Kucharski AJ, Lowe R, Eggo RM, et al. (2019) Assessing the performance of
+#' real-time epidemic forecasts: A case study of Ebola in the Western Area
+#' region of Sierra Leone, 2014-15. PLOS Computational Biology 15(2): e1006785.
+#' \doi{10.1371/journal.pcbi.1006785}
+#' @keywords metric
 
+bias_sample <- function(true_values, predictions) {
 
-bias <- function(true_values, predictions) {
-
-  # ============== Error handling ==============
-
-  if (missing(true_values) | missing(predictions)) {
-    stop("true_values or predictions argument missing")
-  }
-
-  n <- length(true_values)
-
-  if (is.data.frame(predictions)) {
-    predictions <- as.matrix(predictions)
-  }
-  if (!is.matrix(predictions)) {
-    msg <- sprintf("'predictions' should be a matrix. Instead `%s` was found",
-                   class(predictions[1]))
-    stop(msg)
-  }
-  if (nrow(predictions) != n) {
-
-    msg <- sprintf("Mismatch: The true values provided have length `%s`, but 'predictions' has `%s` rows.",
-                   n, nrow(predictions))
-    stop(msg)
-  }
-
-  # ============================================
-
-  ## check whether continuous or integer
-  if (all.equal(as.vector(predictions), as.integer(predictions)) != TRUE) {
-    continuous_predictions <- TRUE
-  } else {
-    continuous_predictions <- FALSE
-  }
-
-  n_pred <- ncol(predictions)
+  # check inputs
+  check_true_values(true_values)
+  check_predictions(predictions, true_values, class = "matrix")
+  prediction_type <- get_prediction_type(predictions)
 
   # empirical cdf
-  P_x <- vapply(seq_along(true_values),
-                function(i) {
-                  sum(predictions[i,] <= true_values[i]) / n_pred
-                },
-                .0)
+  n_pred <- ncol(predictions)
+  p_x <- rowSums(predictions <= true_values) / n_pred
 
-  if (continuous_predictions) {
-    res <- 1 - 2 * P_x
+  if (prediction_type == "continuous") {
+    res <- 1 - 2 * p_x
     return(res)
   } else {
     # for integer case also calculate empirical cdf for (y-1)
-    P_xm1 <- vapply(seq_along(true_values),
-                    function(i) {
-                      sum(predictions[i,] <= true_values[i] - 1) / n_pred
-                    },
-                    .0)
+    p_xm1 <- rowSums(predictions <= (true_values - 1)) / n_pred
 
-    res <- 1 - (P_x + P_xm1)
+    res <- 1 - (p_x + p_xm1)
     return(res)
   }
 }
 
-
-
 #' @title Determines Bias of Quantile Forecasts
 #'
 #' @description
@@ -127,15 +88,23 @@ bias <- function(true_values, predictions) {
 #' For quantile forecasts, bias is measured as
 #'
 #' \deqn{
-#' B_t = (1 - 2 \cdot \max \{i | q_{t,i} \in Q_t \land q_{t,i} \leq x_t\}) 1( x_t \leq q_{t, 0.5}) \\
-#' + (1 - 2 \cdot \min \{i | q_{t,i} \in Q_t \land q_{t,i} \geq x_t\}) 1( x_t \geq q_{t, 0.5}),}
+#' B_t = (1 - 2 \cdot \max \{i | q_{t,i} \in Q_t \land q_{t,i} \leq x_t\})
+#' \mathbf{1}( x_t \leq q_{t, 0.5}) \\
+#' + (1 - 2 \cdot \min \{i | q_{t,i} \in Q_t \land q_{t,i} \geq x_t\})
+#'  \mathbf{1}( x_t \geq q_{t, 0.5}),
+#' }{
+#' B_t = (1 - 2 * max(i | q_{t,i} in Q_t and q_{t,i} <= x_t\))
+#' 1( x_t <= q_{t, 0.5}) + (1 - 2 * min(i | q_{t,i} in Q_t and q_{t,i} >= x_t))
+#'  1( x_t >= q_{t, 0.5}),
+#' }
 #'
 #' where \eqn{Q_t} is the set of quantiles that form the predictive
 #' distribution at time \eqn{t}. They represent our
-#' belief about what the true value $x_t$ will be. For consistency, we define
+#' belief about what the true value \eqn{x_t} will be. For consistency, we
+#' define
 #' \eqn{Q_t} such that it always includes the element
-#' \eqn{q_{t, 0} = - \infty$ and $q_{t,1} = \infty}.
-#' \eqn{1()} is the indicator function that is \eqn{1} if the
+#' \eqn{q_{t, 0} = - \infty} and \eqn{q_{t,1} = \infty}.
+#' \eqn{\mathbf{1}()}{1()} is the indicator function that is \eqn{1} if the
 #' condition is satisfied and $0$ otherwise. In clearer terms, \eqn{B_t} is
 #' defined as the maximum percentile rank for which the corresponding quantile
 #' is still below the true value, if the true value is smaller than the
@@ -160,44 +129,49 @@ bias <- function(true_values, predictions) {
 #' of the central prediction interval
 #' @param true_value a single true value
 #' @return scalar with the quantile bias for a single quantile prediction
-#' @author Nikos Bosse \email{nikosbosse@gmail.com}
+#' @author Nikos Bosse \email{nikosbosse@@gmail.com}
 #' @examples
 #'
-#' lower <- c(6341.000, 6329.500, 6087.014, 5703.500,
-#'            5451.000, 5340.500, 4821.996, 4709.000,
-#'            4341.500, 4006.250, 1127.000, 705.500)
+#' lower <- c(
+#'   6341.000, 6329.500, 6087.014, 5703.500,
+#'   5451.000, 5340.500, 4821.996, 4709.000,
+#'   4341.500, 4006.250, 1127.000, 705.500
+#' )
 #'
-#' upper <- c(6341.000, 6352.500, 6594.986, 6978.500,
-#'            7231.000, 7341.500, 7860.004, 7973.000,
-#'            8340.500, 8675.750, 11555.000, 11976.500)
+#' upper <- c(
+#'   6341.000, 6352.500, 6594.986, 6978.500,
+#'   7231.000, 7341.500, 7860.004, 7973.000,
+#'   8340.500, 8675.750, 11555.000, 11976.500
+#' )
 #'
 #' range <- c(0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 95, 98)
 #'
 #' true_value <- 8062
 #'
-#' quantile_bias(lower = lower, upper = upper,
-#'               range = range, true_value = true_value)
-#'
+#' bias_range(
+#'   lower = lower, upper = upper,
+#'   range = range, true_value = true_value
+#' )
 #' @export
-#'
-
-quantile_bias <- function(range, lower, upper,
-                          true_value) {
+#' @keywords metric
 
+bias_range <- function(range, lower, upper,
+                       true_value) {
   lower_predictions <- lower
   upper_predictions <- upper
 
-  if(any(is.na(upper)) | any(is.na(lower))) {
-
+  if (anyNA(upper) | anyNA(lower)) {
     range <- range[!is.na(upper) & !is.na(lower)]
     lower_predictions <- lower[!is.na(lower) & !is.na(upper)]
     upper_predictions <- upper[!is.na(lower) & !is.na(upper)]
 
     # deal with the point forecast case where inputs may be NA
-    if (length(range) == 0 | length(lower_predictions) == 0 | length(upper_predictions) == 0) {
+    if (length(range) == 0 |
+        length(lower_predictions) == 0 |
+        length(upper_predictions) == 0
+      ) {
       return(NA_real_)
     }
-
   }
 
   # convert range to quantiles
@@ -210,8 +184,8 @@ quantile_bias <- function(range, lower, upper,
     median_prediction <- upper_predictions[range == 0]
   } else {
     median_prediction <-
-      0.5 * upper_predictions[range == min(range)] +
-      0.5 * lower_predictions[range == min(range)]
+      0.5 * upper_predictions[which.min(range)] +
+      0.5 * lower_predictions[which.min(range)]
   }
 
 
@@ -230,10 +204,114 @@ quantile_bias <- function(range, lower, upper,
     lower <- max(lower_quantiles[lower_predictions <= true_value])
     bias <- 1 - 2 * lower
     return(bias)
-  } else if (any(upper_predictions <= true_value)){
+  } else if (any(upper_predictions <= true_value)) {
     upper <- min(upper_quantiles[upper_predictions >= true_value])
     bias <- 1 - 2 * upper
     return(bias)
   }
 }
 
+#' @title Determines Bias of Quantile Forecasts
+#'
+#' @description
+#' Determines bias from quantile forecasts. For an increasing number of
+#' quantiles this measure converges against the sample based bias version
+#' for integer and continuous forecasts.
+#'
+#' @details
+#' For quantile forecasts, bias is measured as
+#'
+#' \deqn{
+#' B_t = (1 - 2 \cdot \max \{i | q_{t,i} \in Q_t \land q_{t,i} \leq x_t\})
+#'  1( x_t \leq q_{t, 0.5}) \\
+#' + (1 - 2 \cdot \min \{i | q_{t,i} \in Q_t \land q_{t,i} \geq x_t\})
+#'  1( x_t \geq q_{t, 0.5}),}
+#'
+#' where \eqn{Q_t} is the set of quantiles that form the predictive
+#' distribution at time \eqn{t}. They represent our
+#' belief about what the true value $x_t$ will be. For consistency, we define
+#' \eqn{Q_t} such that it always includes the element
+#' \eqn{q_{t, 0} = - \infty$ and $q_{t,1} = \infty}.
+#' \eqn{1()} is the indicator function that is \eqn{1} if the
+#' condition is satisfied and $0$ otherwise. In clearer terms, \eqn{B_t} is
+#' defined as the maximum percentile rank for which the corresponding quantile
+#' is still below the true value, if the true value is smaller than the
+#' median of the predictive distribution. If the true value is above the
+#' median of the predictive distribution, then $B_t$ is the minimum percentile
+#' rank for which the corresponding quantile is still larger than the true
+#' value. If the true value is exactly the median, both terms cancel out and
+#' \eqn{B_t} is zero. For a large enough number of quantiles, the
+#' percentile rank will equal the proportion of predictive samples below the
+#' observed true value, and this metric coincides with the one for
+#' continuous forecasts.
+#'
+#' Bias can assume values between
+#' -1 and 1 and is 0 ideally.
+#' @param predictions vector of length corresponding to the number of quantiles
+#' that holds predictions
+#' @param quantiles vector of corresponding size with the quantiles for which
+#' predictions were made
+#' @inheritParams bias_range
+#' @return scalar with the quantile bias for a single quantile prediction
+#' @author Nikos Bosse \email{nikosbosse@@gmail.com}
+#' @examples
+#'
+#' predictions <- c(
+#'   705.500, 1127.000, 4006.250, 4341.500, 4709.000, 4821.996,
+#'   5340.500, 5451.000, 5703.500, 6087.014, 6329.500, 6341.000,
+#'   6352.500, 6594.986, 6978.500, 7231.000, 7341.500, 7860.004,
+#'   7973.000, 8340.500, 8675.750, 11555.000, 11976.500
+#' )
+#'
+#' quantiles <- c(0.01, 0.025, seq(0.05, 0.95, 0.05), 0.975, 0.99)
+#'
+#' true_value <- 8062
+#'
+#' bias_quantile(predictions, quantiles, true_value = true_value)
+#' @export
+#' @keywords metric
+
+bias_quantile <- function(predictions, quantiles, true_value) {
+  # check that predictions and quantiles have the same length
+  if (!length(predictions) == length(quantiles)) {
+    stop("predictions and quantiles must have the same length")
+  }
+
+  if (anyNA(predictions)) {
+    quantiles <- quantiles[!is.na(predictions)]
+    predictions <- predictions[!is.na(predictions)]
+  }
+  # if there is no input, return NA
+  if (length(quantiles) == 0 | length(predictions) == 0) {
+    return(NA_real_)
+  }
+
+  if (0.5 %in% quantiles) {
+    median_prediction <- predictions[quantiles == 0.5]
+  } else {
+    # if median is not available, compute as mean of two innermost quantiles
+    median_prediction <-
+      0.5 * predictions[quantiles == max(quantiles[quantiles < 0.5])] +
+      0.5 * predictions[quantiles == min(quantiles[quantiles > 0.5])]
+  }
+
+  if (true_value == median_prediction) {
+    bias <- 0
+    return(bias)
+  } else if (true_value < median_prediction) {
+    if (true_value < min(predictions)) {
+      bias <- 1
+    } else {
+      q <- max(quantiles[predictions <= true_value])
+      bias <- 1 - 2 * q
+    }
+  } else if (true_value > median_prediction) {
+    if (true_value > max(predictions)) {
+      bias <- -1
+    } else {
+      q <- min(quantiles[predictions >= true_value])
+      bias <- 1 - 2 * q
+    }
+  }
+  return(bias)
+}
